//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
//
//	File: WinMain.cpp
//
//	Author: Brand Gamblin
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/

#ifndef WINMAIN_H
#include "WinMain.h"
#endif

Poly		MyRect;
D3DObj		MyD3D;

//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
//
//	D3DObj
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
void D3DObj::Clear()
{
    SAFE_RELEASE( m_pd3dDevice);
	SAFE_RELEASE( m_pD3D);
}

void D3DObj::Init( HWND hWnd)
{
	m_pD3D = Direct3DCreate8( D3D_SDK_VERSION );
	
	D3DDISPLAYMODE d3ddm;
	m_pD3D->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm );
	
	D3DPRESENT_PARAMETERS d3dpp; 
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed   = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;
    d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	
	m_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING,
		&d3dpp, &m_pd3dDevice );
	
	m_pd3dDevice->SetRenderState ( D3DRS_CULLMODE, D3DCULL_NONE);
	m_pd3dDevice->SetRenderState ( D3DRS_LIGHTING, FALSE);
	m_pd3dDevice->SetRenderState ( D3DRS_ZENABLE, TRUE);
	
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
	m_pd3dDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);//D3DTOP_DISABLE );
	
    // Set diffuse blending for alpha set in vertices.
    m_pd3dDevice->SetRenderState( D3DRS_ALPHABLENDENABLE,   TRUE );
    m_pd3dDevice->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
    m_pd3dDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
	
	m_pd3dDevice->SetVertexShader( D3DFVF_CUSTOMVERTEX );
}

int D3DObj::Render()
{
	// Begin the scene.
	m_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0L );
	m_pd3dDevice->BeginScene();
	
	// Rendering of scene objects happens here.
	
	m_pd3dDevice->SetTexture( 0, MyRect.m_Tex.pTexture );
	
	m_pd3dDevice->SetStreamSource( 0, MyRect.m_VertexBuffer, sizeof(CUSTOMVERTEX) );
	
	m_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP , 0, 2 );
	
	// End the scene.
	m_pd3dDevice->EndScene();
	
	m_pd3dDevice->Present( NULL, NULL, NULL, NULL );
	return 0;
}

//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
//
//	Poly
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
void Poly::Clear()
{
	SAFE_RELEASE(m_VertexBuffer);
}

void Poly::Init ()
{
	if( FAILED( MyD3D.m_pd3dDevice->CreateVertexBuffer( 4*sizeof(CUSTOMVERTEX), 0, D3DFVF_CUSTOMVERTEX, D3DPOOL_DEFAULT, &m_VertexBuffer ) ) )
		return ;
	
	CUSTOMVERTEX* pVertices;
	if( FAILED( m_VertexBuffer->Lock( 0, 4*sizeof(CUSTOMVERTEX), (BYTE**)&pVertices, 0 ) ) )
		return ;
	
	pVertices[0].position=D3DXVECTOR3( -1.0f,  1.0f, 0.5f );
	pVertices[0].color=0xffffffff;
	pVertices[0].tu=0.0;
	pVertices[0].tv=0.0;
	pVertices[1].position=D3DXVECTOR3( 1.0f,  1.0f, 0.5f );
	pVertices[1].color=0xffffffff;
	pVertices[1].tu=1.0;
	pVertices[1].tv=0.0;
	pVertices[2].position=D3DXVECTOR3( -1.0f,  -1.0f, 0.5f );
	pVertices[2].color=0xffffffff;
	pVertices[2].tu=0.0;
	pVertices[2].tv=1.0;
	pVertices[3].position=D3DXVECTOR3(  1.0f,  -1.0f, 0.5f );
	pVertices[3].color=0xffffffff;
	pVertices[3].tu=1.0;
	pVertices[3].tv=1.0;
	
	m_Tex.Init ();
	
	m_VertexBuffer->Unlock();
}

void Poly::SetVertRect(int x, int y, int w, int h, float d)
{
	float left, top, right, bottom;
	left	= ((float)x / (float)VIEWPORT_WIDTH) * 2.0f - 1.0f;
	right	= ((float)(x + w) / (float)VIEWPORT_WIDTH) * 2.0f - 1.0f;
	top		= ((float)y / (float)VIEWPORT_HEIGHT) * 2.0f - 1.0f;
	bottom	= ((float)(y + h) / (float)VIEWPORT_HEIGHT) * 2.0f - 1.0f;
	
	CUSTOMVERTEX* pVertices;
	if( FAILED( m_VertexBuffer->Lock( 0, 4*sizeof(CUSTOMVERTEX), (BYTE**)&pVertices, 0 ) ) )
		return ;
	
	pVertices[0].position=D3DXVECTOR3( left,  bottom, d );
	pVertices[1].position=D3DXVECTOR3( right,  bottom, d );
	pVertices[2].position=D3DXVECTOR3( left,  top, d );
	pVertices[3].position=D3DXVECTOR3( right,  top, d );
	
	m_VertexBuffer->Unlock();
}

//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
//
//	Texture
//
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
void Texture::Clear()
{
	SAFE_RELEASE(pTexture);	
}

void Texture::Init ()
{
	if (!MyD3D.m_pd3dDevice) return;
	
	int alpha = 255; // fully opaque

	m_Width=256;
	m_Height=256;
	m_Pitch=256*4;
	
	MyD3D.m_pd3dDevice->CreateTexture( m_Width, m_Height, 0, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, &pTexture );
	D3DSURFACE_DESC pDesc;
	pTexture->GetLevelDesc(0, &pDesc );
	
    D3DLOCKED_RECT d3dlr;
    pTexture->LockRect( 0, &d3dlr, 0, 0 );
	DWORD * pDst = (DWORD *)d3dlr.pBits;
	int DPitch = d3dlr.Pitch/4;

	for (int y=0; y<m_Height; ++y)
		for (int x=0; x<m_Width; ++x)
			pDst[y*DPitch + x] = (alpha << 24 | (255 - x ) << 16 | x << 8 | y );

		pTexture->UnlockRect (0);
}

//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/
// 
// Normal Windows Functions
// 
//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\/

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return 0;
	}
	return DefWindowProc( hWnd, msg, wParam, lParam );
}

INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class.
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
		GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
		"D2D Tutorial", NULL };
    RegisterClassEx( &wc );
	
    // Create the application's window.
    HWND hWnd = CreateWindow( "D2D Tutorial", "D2D Tutorial", WS_OVERLAPPEDWINDOW, 100, 100, VIEWPORT_WIDTH, VIEWPORT_HEIGHT,
		GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
    // Initialize Direct3D
	MyD3D.Init (hWnd);

	MyRect.Init ();
	
	// Show the window
	ShowWindow( hWnd, SW_SHOWDEFAULT );
	UpdateWindow( hWnd );
	
	// Enter the message loop
	MSG msg;
	ZeroMemory( &msg, sizeof(msg) );
	while( msg.message!=WM_QUIT )
	{
		if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
			MyD3D.Render();
	}

	return 0;
}
   
